#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
This calculates the fractional occupancy for a phase. This calculates the
solvent contribution from the given file containing solvent xml, and adds the
contribution for the single solute SSIP at the given concentration.

@author: mark
"""

import logging
import solventmapcreator.io.solventxmlreader as solventxmlreader

logging.basicConfig()
LOGGER = logging.getLogger(__name__)
LOGGER.setLevel(logging.WARN)

def calculate_fractional_occupancy_for_phase(solvent_filename, solvent_id, solute_concentration):
    """This calculates the fractional occupancy of the phase. This is the sum
    of the solvent and solute contributions.
    """
    solvent_fractional_occupancy = get_fractional_occupancy_for_solvent_from_file(solvent_filename, solvent_id)
    solute_fractional_occupancy = calculate_fractional_occupancy_contribution_from_solute(solute_concentration)
    return solute_fractional_occupancy + solvent_fractional_occupancy

def calculate_fractional_occupancy_contribution_from_solute(solute_concentration):
    """This calculates the fractional occupancy contribution for the
    solute concentration given. This assumes teh solute is a single SSIP.
    """
    return solute_concentration/solventxmlreader.IDEAL_MAX_SSIP_CONCENTRATION_MOLAR

def get_fractional_occupancy_for_solvent_from_file(solvent_filename, solvent_id):
    """This reads in the file and gets the fractional occupancy contribution
    from the solvent.
    """
    return solventxmlreader.get_fractional_occupancy_for_solvent_from_file(solvent_filename, solvent_id)
