#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Thu Jul  6 21:13:47 2017

@author: mark
"""

import logging
import unittest
import numpy as np
import solventmapcreator.polynomialanalysis.polynomialdataanalysis as polynomialdataanalysis

logging.basicConfig()
LOGGER = logging.getLogger(__name__)
LOGGER.setLevel(logging.WARN)

class PolynomialDataAnalysisTestCase(unittest.TestCase):
    """Test case for the polynomialdataanalysis script
    """
    def setUp(self):
        """set up for class.
        """
        self.expected_data_by_solvent_id = {"expected_poly_fit":{1:{"coefficients":np.array([-0.05, 
                                                                                             1.07142857]),
                                                                    "RMSE":0.0154303,
                                                                    "order":1,
                                                                    "covar":0.0007143},
                                                                 2:{"coefficients":np.array([-0.05, 
                                                                                             1.07142857, 0.1]),
                                                                    "RMSE":0.0254303,
                                                                    "order":1,
                                                                    "covar":0.0008143}},
                                            "expected_poly_fit2":{1:{"coefficients":np.array([-0.05, 
                                                                                             1.07142857]),
                                                                    "RMSE":0.0254303,
                                                                    "order":1,
                                                                    "covar":0.007143},
                                                                 2:{"coefficients":np.array([-0.05, 
                                                                                             1.07142857, 0.1]),
                                                                    "RMSE":0.0354303,
                                                                    "order":1,
                                                                    "covar":0.0008143}}}
        self.rmse_dict = polynomialdataanalysis.get_best_poly_fit_rmse(self.expected_data_by_solvent_id)
        self.covar_dict = polynomialdataanalysis.get_best_poly_fit_covar(self.expected_data_by_solvent_id)
    def tearDown(self):
        """tear down for tests.
        """
        del self.expected_data_by_solvent_id
        del self.rmse_dict
        del self.covar_dict
    def test_get_total_sorted_by_alt_metric(self):
        """Test to see if 
        """
        rmse_solvent_id_dict = polynomialdataanalysis.get_solvent_id_by_order(self.rmse_dict)
        covar_solvent_id_dict = polynomialdataanalysis.get_solvent_id_by_order(self.covar_dict)
        ordered_id_dict = polynomialdataanalysis.generate_lists_by_alt_metric(rmse_solvent_id_dict, covar_solvent_id_dict)
        expected_tuple = ((1, 2),((1, 0),(1, 0)))
        actual_tuple = polynomialdataanalysis.get_total_sorted_by_alt_metric([1, 2], ordered_id_dict)
        self.assertEqual(expected_tuple, actual_tuple)
    def test_get_total_by_fit_order(self):
        """Test to see if expected totals are returned.
        """
        rmse_solvent_id_dict = polynomialdataanalysis.get_solvent_id_by_order(self.rmse_dict)
        covar_solvent_id_dict = polynomialdataanalysis.get_solvent_id_by_order(self.covar_dict)
        expected_tuple_rmse = ((1,), (2,))
        actual_tuple_rmse = polynomialdataanalysis.get_total_by_fit_order(rmse_solvent_id_dict)
        self.assertEqual(expected_tuple_rmse, actual_tuple_rmse)
        expected_tuple_covar = ((1, 2), (1, 1))
        actual_tuple_covar = polynomialdataanalysis.get_total_by_fit_order(covar_solvent_id_dict)
        self.assertEqual(expected_tuple_covar, actual_tuple_covar)
    def test_generate_lists_by_alt_metric(self):
        """Test to see if expected dict is produced.
        """
        expected_dict = {1:{1: ['expected_poly_fit'], 2: ['expected_poly_fit2']}}
        rmse_solvent_id_dict = polynomialdataanalysis.get_solvent_id_by_order(self.rmse_dict)
        covar_solvent_id_dict = polynomialdataanalysis.get_solvent_id_by_order(self.covar_dict)
        actual_dict = polynomialdataanalysis.generate_lists_by_alt_metric(rmse_solvent_id_dict, covar_solvent_id_dict)
        self.assertDictEqual(expected_dict, actual_dict)
    def test_extract_lists_by_order_of_alt_metric(self):
        """Test to if expected dict is produced.
        """
        expected_dict = {1: ['expected_poly_fit'], 2: ['expected_poly_fit2']}
        rmse_list = ['expected_poly_fit', 'expected_poly_fit2']
        covar_solvent_id_dict = polynomialdataanalysis.get_solvent_id_by_order(self.covar_dict)
        actual_dict = polynomialdataanalysis.extract_lists_by_order_of_alt_metric(rmse_list, covar_solvent_id_dict)
        self.assertDictEqual(expected_dict, actual_dict)
    def test_get_solvent_id_by_order(self):
        """Test to see if expected dictionary is returned
        """
        expected_dict = {1: ['expected_poly_fit', 'expected_poly_fit2']}
        actual_dict = polynomialdataanalysis.get_solvent_id_by_order(self.rmse_dict)
        self.assertDictEqual(expected_dict, actual_dict)
    def test_get_solvent_id(self):
        """Test to see if expected list is returned.
        """
        expected_list = ['expected_poly_fit', 'expected_poly_fit2']
        actual_list = polynomialdataanalysis.get_solvent_id(self.rmse_dict[1])
        self.assertListEqual(expected_list, actual_list)
    def test_get_best_poly_fit_rmse(self):
        """Test to see if expected dict is produced.
        """
        expected_dict = {1: [('expected_poly_fit', 0.0154303), ('expected_poly_fit2', 0.0254303)]}
        actual_dict = polynomialdataanalysis.get_best_poly_fit_rmse(self.expected_data_by_solvent_id)
        self.assertDictEqual(expected_dict, actual_dict)
    def test_extract_best_poly_fit_rmse(self):
        """Test to see if expected value is extracted.
        """
        expected_order = 1
        expected_rmse = 0.0154303
        actual_order, actual_rmse = polynomialdataanalysis.extract_best_poly_fit_rmse(self.expected_data_by_solvent_id["expected_poly_fit"])
        self.assertAlmostEqual(expected_rmse, actual_rmse)
        self.assertEqual(expected_order, actual_order)
    def test_get_best_poly_fit_covar(self):
        """Test to see if expected dict is produced.
        """
        expected_dict = {1: [('expected_poly_fit', 0.0007143)], 2: [('expected_poly_fit2', 0.0008143)]}
        actual_dict = polynomialdataanalysis.get_best_poly_fit_covar(self.expected_data_by_solvent_id)
        self.assertDictEqual(expected_dict, actual_dict)
    def test_extract_best_poly_fit_covar(self):
        """Test to see if expected value is extracted.
        """
        expected_order = 1
        expected_covar = 0.0007143
        actual_order, actual_covar = polynomialdataanalysis.extract_best_poly_fit_covar(self.expected_data_by_solvent_id["expected_poly_fit"])
        self.assertAlmostEqual(expected_covar, actual_covar)
        self.assertEqual(expected_order, actual_order)